@@ -1,12 +1,13 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "citar" "1.4.0"
+(define-package "citar" "20251109.1138"
   "Citation-related commands for org, latex, markdown."
   '((emacs    "27.1")
     (parsebib "4.2")
     (org      "9.5")
-    (citeproc "0.9"))
+    (citeproc "0.9")
+    (compat   "30"))
   :url "https://github.com/emacs-citar/citar"
-  :commit "e21bf22b29d8ca40649517bb7dc503765f240282"
-  :revdesc "v1.4.0-0-ge21bf22b29d8"
+  :commit "dc7018eb36fb3540cb5b7fc526d6747144437eef"
+  :revdesc "dc7018eb36fb"
   :authors '(("Bruce D'Arcus" . "https://github.com/bdarcus"))
   :maintainers '(("Bruce D'Arcus" . "https://github.com/bdarcus")))
