@@ -6,10 +6,10 @@
 ;; Maintainer: Bruce D'Arcus <https://github.com/bdarcus>
 ;; Created: February 27, 2021
 ;; SPDX-License-Identifier: GPL-3.0-or-later
-;; Package-Version: 1.4.0
-;; Package-Revision: v1.4.0-0-ge21bf22b29d8
+;; Package-Version: 20251109.1138
+;; Package-Revision: dc7018eb36fb
 ;; Homepage: https://github.com/emacs-citar/citar
-;; Package-Requires: ((emacs "27.1") (parsebib "4.2") (org "9.5") (citeproc "0.9"))
+;; Package-Requires: ((emacs "27.1") (parsebib "4.2") (org "9.5") (citeproc "0.9") (compat "30"))
 
 ;; This file is not part of GNU Emacs.
 ;;
@@ -21,11 +21,13 @@
 ;;
 ;;; Code:
 
+(require 'compat)
 (eval-when-compile
   (require 'cl-lib)
   (require 'subr-x))
 (require 'seq)
 (require 'map)
+(require 'bibtex)
 (require 'browse-url)
 (require 'citar-cache)
 (require 'citar-format)
@@ -92,6 +94,11 @@ buffer.")
   :group 'citar
   :type '(repeat directory))
 
+(defcustom citar-library-paths-recursive nil
+  "Whether library paths should be searched recursively."
+  :group 'citar
+  :type 'boolean)
+
 (defcustom citar-library-file-extensions nil
   "List of file extensions to filter for related files.
 
@@ -191,16 +198,14 @@ references as a string."
 
 (defcustom citar-display-transform-functions
   ;; TODO change this name, as it might be confusing?
-  `((sn . (citar--shorten-names))
+  '((sn . (citar--shorten-names))
     (etal . (citar--shorten-names 3 "&")))
   "Configure transformation of field display values from raw values.
 
 When the car symbol is associated with a field, run the cdr function
 and optional arguments on the string value."
   :group 'citar
-  :type '(alist :key-type symbol
-          ;; REVIEW is this OK for now?
-          :value-type list))
+  :type '(alist :key-type symbol :value-type sexp))
 
 ;; Indicator defstruct
 
@@ -341,6 +346,11 @@ always prompt to select."
                (function-item citar-open-notes)
                (function-item citar-open-note))))
 
+(defcustom citar-bibtex-no-export-fields nil
+  "A list of fields that should be ignored when exporting BibTeX entries."
+  :group 'citar
+  :type '(repeat string))
+
 ;;;; File, note, and URL handling
 
 (defcustom citar-open-resources '(:files :links :notes :create-notes)
@@ -584,6 +594,7 @@ When nil, all citar commands will use `completing-read'."
 
 (defvar citar-map
   (let ((map (make-sparse-keymap)))
+    (define-key map (kbd "a") #'citar-add-file-to-library)
     (define-key map (kbd "c") #'citar-insert-citation)
     (define-key map (kbd "k") #'citar-insert-keys)
     (define-key map (kbd "r") #'citar-copy-reference)
@@ -849,7 +860,7 @@ CATEGORY is one of:
 
 (defun citar--annotate-note (candidate)
   "Annotate note CANDIDATE."
-  (when-let (((eq 'note (get-text-property 0 'citar--resource candidate)))
+  (when-let* (((eq 'note (get-text-property 0 'citar--resource candidate)))
              (annotate (citar--get-notes-config :annotate)))
     (funcall annotate (substring-no-properties candidate))))
 
@@ -863,10 +874,10 @@ Return nil if there are no resources.
 Use `completing-read' to prompt for a resource, unless there is
 only one resource and `citar-open-prompt' is t or contains
 `this-command'. Return nil if the user declined to choose."
-  (when-let ((resources (citar--get-resource-candidates citekeys :files files :links links
+  (when-let* ((resources (citar--get-resource-candidates citekeys :files files :links links
                                                         :notes notes :create-notes create-notes)))
     (pcase-let ((`(,category . ,cands) resources))
-      (when-let ((selected
+      (when-let* ((selected
                   (if (not (or (cdr cands) (eq t citar-open-prompt) (memq this-command citar-open-prompt)))
                       (car cands)
                     (let* ((metadata `(metadata
@@ -963,9 +974,9 @@ visible in the completion UI."
 Return a hash table with the keys being completion candidate
 strings and values being citation keys.
 
-Return nil if `citar-bibliographies' returns nil."
+Return nil if `citar-bibliography' returns nil."
   ;; Populate bibliography cache.
-  (when-let ((bibs (citar--bibliographies)))
+  (when-let* ((bibs (citar--bibliographies)))
     (let* ((citar--entries (citar-cache--entries bibs))
            (preformatted (citar-cache--preformatted bibs))
            (indicatorprocs (citar--make-indicator-processors citar-indicators))
@@ -1090,7 +1101,7 @@ was found to have a value, and VALUE is its value."
                    (citar-get-entry citekey-or-entry)
                  citekey-or-entry)))
     (seq-some (lambda (field)
-                (when-let ((value (citar-get-value field entry)))
+                (when-let* ((value (citar-get-value field entry)))
                   (cons field value)))
               fields)))
 
@@ -1180,7 +1191,7 @@ nil, return nil."
                              (let (keylinks)
                                (when entry
                                  (pcase-dolist (`(,fieldname . ,urlformat) citar-link-fields)
-                                   (when-let ((fieldvalue (citar-get-value fieldname entry)))
+                                   (when-let* ((fieldvalue (citar-get-value fieldname entry)))
                                      (push (format urlformat fieldvalue) keylinks))))
                                (nreverse keylinks)))))))
 
@@ -1201,7 +1212,7 @@ files.
 
 For example, to test whether CITEKEY has associated files:
 
-  (when-let ((hasfilesp (citar-has-files)))
+  (when-let* ((hasfilesp (citar-has-files)))
     (funcall hasfilesp CITEKEY))
 
 When testing many citekeys, call this function once and use the
@@ -1212,7 +1223,7 @@ check any bibliography entries that are cross-referenced from the
 given CITEKEY; see `citar-crossref-variable'."
   (citar--has-resources
    (mapcar (lambda (source)
-             (when-let ((hasitems (plist-get source :hasitems)))
+             (when-let* ((hasitems (plist-get source :hasitems)))
                (funcall hasitems)))
            citar-file-sources)))
 
@@ -1267,7 +1278,7 @@ the returned predicate is nil.
 When `citar-crossref-variable' is the name of a crossref field,
 the returned predicate also tests if an entry cross-references
 another entry in ENTRIES that has associated resources."
-  (when-let ((hasresourcep (if (functionp predicates)
+  (when-let* ((hasresourcep (if (functionp predicates)
                                predicates
                              (let ((predicates (remq nil predicates)))
                                (if (null (cdr predicates))
@@ -1278,10 +1289,10 @@ another entry in ENTRIES that has associated resources."
                                    (seq-some (lambda (predicate)
                                                (funcall predicate citekey))
                                              predicates)))))))
-    (if-let ((xref citar-crossref-variable))
+    (if-let* ((xref citar-crossref-variable))
         (lambda (citekey)
           (or (funcall hasresourcep citekey)
-              (when-let ((xkey (citar-get-value xref citekey)))
+              (when-let* ((xkey (citar-get-value xref citekey)))
                 (funcall hasresourcep xkey))))
       hasresourcep)))
 
@@ -1311,7 +1322,7 @@ CITEKEYS."
     (cl-flet* ((getreslists (citekey) (delq nil (mapcar (apply-partially #'gethash citekey) resources)))
                (xresources (citekey entry) (apply #'append
                                                   (nconc (getreslists citekey)
-                                                         (when-let ((xkey (and xref
+                                                         (when-let* ((xkey (and xref
                                                                                (citar-get-value xref entry))))
                                                            (getreslists xkey))))))
       (citar--get-resources-using-function #'xresources citekeys))))
@@ -1328,7 +1339,7 @@ Note: This is a helper function to make it easier to write
 getters for file, note, and link resources."
   (let ((resources (make-hash-table :test 'equal)))
     (prog1 resources
-      (cl-flet ((putresult (citekey entry) (when-let ((result (funcall func citekey entry)))
+      (cl-flet ((putresult (citekey entry) (when-let* ((result (funcall func citekey entry)))
                                              (puthash citekey result resources))))
         (if (null citekeys)
             (maphash #'putresult (citar-get-entries))
@@ -1347,9 +1358,10 @@ it.
 
 SOURCE-PLIST must be as specified in the documentation of
 `citar-add-file-sources'."
-  (let* ((directory (if (cdr citar-library-paths)
-                        (completing-read "Directory: " citar-library-paths)
-                      (car citar-library-paths)))
+  (let* ((dirs (citar-file--library-dirs))
+         (directory (if (cdr dirs)
+                        (completing-read "Directory: " dirs)
+                      (car dirs)))
          (extension (or (plist-get source-plist :extension)
                         (read-string "File extension: ")))
          (destfile (expand-file-name citekey directory))
@@ -1509,7 +1521,7 @@ replace last comma."
   (interactive)
   (unless (minibufferp)
     (user-error "Command can only be used in minibuffer"))
-  (when-let ((enable-recursive-minibuffers t)
+  (when-let* ((enable-recursive-minibuffers t)
              (search (completing-read "Preset: " citar-presets)))
     (insert search)))
 
@@ -1544,7 +1556,7 @@ specifying TYPE."
   ;; it relies on RESOURCE having the `citar--resource' text property to decide which action to take. However,
   ;; `embark-act' strips text properties for interactive commands, for which it injects the target text as
   ;; minibuffer input.
-  (if-let ((opener (pcase (or type (get-text-property 0 'citar--resource resource))
+  (if-let* ((opener (pcase (or type (get-text-property 0 'citar--resource resource))
                      ('file #'citar-file-open)
                      ('url #'browse-url)
                      ('note #'citar-open-note)
@@ -1629,7 +1641,7 @@ to open from a list of all notes."
 (defun citar-open-entry-in-file (citekey)
   "Open entry for CITEKEY."
   ;; Adapted from 'bibtex-completion-show-entry'.
-  (when-let ((bib-files (citar--bibliography-files)))
+  (when-let* ((bib-files (citar--bibliography-files)))
     (catch 'break
       (dolist (bib-file bib-files)
         (let ((buf (or (get-file-buffer bib-file)
@@ -1670,6 +1682,12 @@ including the citekeys, is maintained in Zotero with Better BibTeX."
             (dolist (bib-file bibtex-files)
               (insert-file-contents bib-file))
             (bibtex-search-entry citekey)
+            (dolist (field citar-bibtex-no-export-fields)
+              (let ((position (bibtex-search-forward-field
+                               field t)))
+                (when position
+                  (delete-region (caar position)
+                                 (car (last position))))))
             (let ((beg (bibtex-beginning-of-entry))
                   (end (bibtex-end-of-entry)))
               (buffer-substring-no-properties beg end)))))
@@ -1787,7 +1805,7 @@ URL."
 (defun citar-dwim ()
   "Run the default action on citation keys found at point."
   (interactive)
-  (if-let ((citekeys (or (citar-key-at-point) (citar-citation-at-point))))
+  (if-let* ((citekeys (or (citar-key-at-point) (citar-citation-at-point))))
       (citar-run-default-action (if (listp citekeys) citekeys (list citekeys)))
     (user-error "No citation keys found")))
 
@@ -1836,17 +1854,17 @@ SOURCE must be a plist representing a notes source with NAME. See
   (let ((required '(:items :hasitems :open))
         (optional '(:name :category :create :transform :annotate))
         (keys (map-keys config)))
-    (when-let ((missing (cl-set-difference required keys)))
+    (when-let* ((missing (cl-set-difference required keys)))
       (error "Note source `%s' missing required keys: %s" name missing))
-    (when-let ((extra (cl-set-difference keys (append required optional))))
+    (when-let* ((extra (cl-set-difference keys (append required optional))))
       (warn "Note source `%s' has unknown keys: %s" name extra)))
 
   (pcase-dolist (`(,type . ,props)
                  '((functionp :items :hasitems :open :create :transform :annotate)
                    (stringp :name)
                    (symbolp :category)))
-    (when-let ((wrongtype (seq-filter (lambda (prop)
-                                        (when-let ((value (plist-get config prop)))
+    (when-let* ((wrongtype (seq-filter (lambda (prop)
+                                        (when-let* ((value (plist-get config prop)))
                                           (not (funcall type value)))) props)))
       (error "Note source `%s' keys must be of type %s: %s" name type wrongtype))))
 
